@@ -7,7 +7,7 @@ use diagnostics;
 require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw( auction );
-our $VERSION = '0.46';
+our $VERSION = '0.50';
 
 #Variables global to the package	
 my $maximize_total_benefit;
@@ -15,12 +15,9 @@ my $matrix_spaces;     # used to print messages on the screen
 my $decimals;          # the number of digits after the decimal point
 my $max_matrix_value;
 my $need_transpose = 0;
-my $inicial_price;
-my $iter_count_global = 0;
-my $iter_count_local  = 0;
-my ( $array1_size, $array2_size, $min_size, $max_size, $original_max_size, $epsilon_scaling );
-my ( %assignned_object, %assignned_person, %price_object, %objects_desired_by_this );
-my %index_correlation;
+my ( $array1_size, $array2_size, $min_size, $max_size, $original_max_size );
+my ( $inicial_price, $iter_count_global, $iter_count_local, $epsilon_scaling );
+my ( %index_correlation, %assignned_object, %assignned_person, %price_object, %objects_desired_by_this );
 
 sub auction { #							=> default values
 	my %args = ( matrix_ref				=> undef,     # reference to array: matrix N x M			                                     
@@ -31,6 +28,13 @@ sub auction { #							=> default values
 				@_,                                   # argument pair list goes here
 				);
        
+	$max_matrix_value = 0;
+	$iter_count_global = 0;
+	$epsilon_scaling = 0;
+	%index_correlation = ();
+	%price_object = ();
+	%objects_desired_by_this = ();	
+	
 	my @matrix_input = @{$args{matrix_ref}};          # Input: Reference to the input matrix (NxM) = $min_size x $max_size
    
 	$array1_size = $#matrix_input + 1;
@@ -71,8 +75,8 @@ sub auction { #							=> default values
 	delete_multiple_columns( \@matrix, $args{verbose} ) if ( $min_size >= 2 and $min_size != $max_size );
      
 	# epsilon is the stepsize and auction algorithm terminates with a feasible assignment if the problem data are integer and epsilon < 1/min(N,M).
-	# There is a trade-off between runtime and the chosen stepsize. Using the largest possible increment accelerates the algorithm.
-   
+	# There is a trade-off between runtime and the chosen stepsize. Using the largest possible increment accelerates the algorithm.  
+	
 	$inicial_price    = $args{inicial_price};
 	$price_object{$_} = $inicial_price for ( 0 .. $max_size - 1 );
    
@@ -97,7 +101,7 @@ sub auction { #							=> default values
 		while ( (scalar keys %assignned_person) < $max_size ){ # while there is at least one element not assigned.
          
 			$iter_count_global++;
-			$iter_count_local++;		
+			$iter_count_local++;
 			
 			auctionRound( \@matrix, $epsilon, $args{verbose} );
 		 
@@ -210,7 +214,7 @@ sub delete_multiple_columns { # if the column elements do not change the final r
       for my $i ( 0 .. $#{$matrix_ref} ) {
          print " [";
          for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
-            printf(" %2.0f", $matrix_ref->[$i]->[$j] );
+            printf(" %${matrix_spaces}.${decimals}f", $matrix_ref->[$i]->[$j] );
 			if ( defined $intersection_columns{$j} and $intersection_columns{$j} == $min_size ){ print "**"; } else{ print "  "; }
          }
          print "]\n";
@@ -242,7 +246,7 @@ sub delete_multiple_columns { # if the column elements do not change the final r
 		for my $i ( 0 .. $#{$matrix_ref} ) {
 			print " [";
 			for my $j ( 0 .. $#{$matrix_ref->[$i]} ) {
-				printf(" %2.0f  ", $matrix_ref->[$i]->[$j] );
+				printf(" %${matrix_spaces}.${decimals}f  ", $matrix_ref->[$i]->[$j] );
 			}
 			print "]\n";
 		}
@@ -435,53 +439,92 @@ sub auctionRound {
 			##  |  person 2          price_2_0
 			##  |  ...
 			##  i  person (N - 1)    price_i_0	
-			
+
 			my ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );
 			my ( $Opt01ValForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI ) = ( -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value), -10 * exp ($max_matrix_value) );
-			my ( $bid01ForPersonI, $bid02ForPersonI );
-            
-			# The working sets @objects_with_greater_benefits are updated dynamically by $objects_desired_by_this{$person} considering the price update of objects.
-			# For each person, the Current Value $curVal of the most desired objects contained in @objects_with_greater_benefits tend to converge to a specific value.
+			my $BadValForPersonI = 10 * exp ($max_matrix_value);
+			my ( $bid01ForPersonI, $bid02ForPersonI );			
+			my %current_value;
+			my @updated_price;
 			
-			my @objects_with_greater_benefits = keys %{$objects_desired_by_this{$person}}; # sort { $a <=> $b }			
-			my @updated_price = grep { $objects_desired_by_this{$person}{$_} == $price_object{$_} } @objects_with_greater_benefits;
+			$seen_ghost++ if ( not defined $matrix[$person] and $min_size < $max_size );
+
+			# The @objects_with_greater_benefits list are updated dynamically by $objects_desired_by_this{$person} by considering the current price (or current value) of objects.		
+			# For each person, the Current Value $current_value{$object} of the most desired objects contained in the @objects_with_greater_benefits list tends to converge to a specific value.
+			
+			my @objects_with_greater_benefits = keys %{$objects_desired_by_this{$person}}; # sort { $a <=> $b }
 			
 			# There is at least one object in the @objects_with_greater_benefits list whose price is updated ? use old list : generate new list;
+			
+			for my $object ( @objects_with_greater_benefits ){ # use old list
+			
+				my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object];
+				$current_value{$object} = $matrix_value - $price_object{$object};
+				
+				push @updated_price, $object if ( $objects_desired_by_this{$person}{$object} == $current_value{$object} );
 
-			for my $object ( @updated_price >= 1 ? @objects_with_greater_benefits : ( 0 .. $max_size - 1 ) )		
-			#for my $object ( 0 .. $max_size - 1 )			
-			{				
-				$seen_ghost++ if ( not defined $matrix[$person] and $min_size < $max_size );
-				
-				my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object] ;
-				
-				my $curVal = $matrix_value - $price_object{$object};
-				
-				if ( $curVal > $Opt01ValForPersonI )
+				if ( $current_value{$object} > $Opt01ValForPersonI )
 				{
 					$Opt03ValForPersonI = $Opt02ValForPersonI;
 					$Opt03ObjForPersonI = $Opt02ObjForPersonI;
 					$Opt02ValForPersonI = $Opt01ValForPersonI;
 					$Opt02ObjForPersonI = $Opt01ObjForPersonI;
-					$Opt01ValForPersonI = $curVal;
+					$Opt01ValForPersonI = $current_value{$object};
 					$Opt01ObjForPersonI = $object;
 				}
-				elsif ( $curVal > $Opt02ValForPersonI )
+				elsif ( $current_value{$object} > $Opt02ValForPersonI )
 				{
 					$Opt03ValForPersonI = $Opt02ValForPersonI;
 					$Opt03ObjForPersonI = $Opt02ObjForPersonI;
-					$Opt02ValForPersonI = $curVal;
+					$Opt02ValForPersonI = $current_value{$object};
 					$Opt02ObjForPersonI = $object;
 				}
-				elsif ( $curVal > $Opt03ValForPersonI )
+				elsif ( $current_value{$object} > $Opt03ValForPersonI )
 				{
-					$Opt03ValForPersonI = $curVal;
+					$Opt03ValForPersonI = $current_value{$object};
 					$Opt03ObjForPersonI = $object;
 				}
-				
+
 				if ( $verbose >= 8 ){
-					printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f \n", $person, $object, $curVal, $person, $object, $matrix_value, $object, $price_object{$object};
-				}				
+					printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f <old list> \n", $person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object} ;
+				}
+			}
+
+			if ( not @updated_price ){ # if all prices are outdated
+		
+				for my $object ( 0 .. $max_size - 1 ) # generate new list
+				{
+					next if ( defined $current_value{$object} );
+					
+					my $matrix_value = $seen_ghost ? 0 : $matrix[$person]->[$object];				
+					$current_value{$object} = $matrix_value - $price_object{$object};
+				
+					if ( $current_value{$object} > $Opt01ValForPersonI )
+					{
+						$Opt03ValForPersonI = $Opt02ValForPersonI;
+						$Opt03ObjForPersonI = $Opt02ObjForPersonI;
+						$Opt02ValForPersonI = $Opt01ValForPersonI;
+						$Opt02ObjForPersonI = $Opt01ObjForPersonI;
+						$Opt01ValForPersonI = $current_value{$object};
+						$Opt01ObjForPersonI = $object;
+					}
+					elsif ( $current_value{$object} > $Opt02ValForPersonI )
+					{
+						$Opt03ValForPersonI = $Opt02ValForPersonI;
+						$Opt03ObjForPersonI = $Opt02ObjForPersonI;
+						$Opt02ValForPersonI = $current_value{$object};
+						$Opt02ObjForPersonI = $object;
+					}
+					elsif ( $current_value{$object} > $Opt03ValForPersonI )
+					{
+						$Opt03ValForPersonI = $current_value{$object};
+						$Opt03ObjForPersonI = $object;
+					}
+
+					if ( $verbose >= 8 ){
+						printf " personI = %3s ; objectJ = %3s ; Current Value %12.5f = \$matrix[%3s][%3s] %3.0f - \$price_object{%3s} %10.5f \n", $person, $object, $current_value{$object}, $person, $object, $matrix_value, $object, $price_object{$object};
+					}				
+				}	
 			}
 			
 			$bid01ForPersonI = $Opt01ValForPersonI - $Opt02ValForPersonI + $epsilon;
@@ -496,15 +539,22 @@ sub auctionRound {
 			
 				$info{$Opt02ObjForPersonI}{$bid02ForPersonI} = $person if ( ++$seen_bid{$Opt01ObjForPersonI}{$bid01ForPersonI} >= 2 );				
 					
-				if ( @updated_price == 0 ){ # if all prices are outdated
-				
-					$objects_desired_by_this{$person}{$_} = $price_object{$_} for ( @objects_with_greater_benefits ); # update prices for all objects in the list
-					
-					delete $objects_desired_by_this{$person} if ( $epsilon_scaling % 3 == 0 and @objects_with_greater_benefits > 3 * $epsilon_scaling ) ; # delete old list
+				if ( not @updated_price ){ # if all prices are outdated						
 
-					$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = $price_object{$Opt01ObjForPersonI}; # information about the most desired objects
-					$objects_desired_by_this{$person}{$Opt02ObjForPersonI} = $price_object{$Opt02ObjForPersonI} if (defined $Opt02ObjForPersonI);
-					$objects_desired_by_this{$person}{$Opt03ObjForPersonI} = $price_object{$Opt03ObjForPersonI} if (defined $Opt03ObjForPersonI);					
+					$objects_desired_by_this{$person}{$_} = $current_value{$_} for ( @objects_with_greater_benefits ); # update current values for all objects in the old list
+					
+					if ( $epsilon_scaling == 6 and $iter_count_local <= $epsilon_scaling and @objects_with_greater_benefits > $epsilon_scaling )
+					{
+						for my $object ( @objects_with_greater_benefits ){
+							$BadValForPersonI = $current_value{$object} if ( $current_value{$object} < $BadValForPersonI );
+						}
+
+						delete $objects_desired_by_this{$person} if ( ($Opt01ValForPersonI - $BadValForPersonI) > 2 * $min_size * $epsilon ); # delete old list
+					}
+
+					$objects_desired_by_this{$person}{$Opt01ObjForPersonI} = $current_value{$Opt01ObjForPersonI}; # information about the most desired objects
+					$objects_desired_by_this{$person}{$Opt02ObjForPersonI} = $current_value{$Opt02ObjForPersonI} if (defined $Opt02ObjForPersonI);
+					$objects_desired_by_this{$person}{$Opt03ObjForPersonI} = $current_value{$Opt03ObjForPersonI} if (defined $Opt03ObjForPersonI);					
 				}
 			}
 			
@@ -512,9 +562,11 @@ sub auctionRound {
 				my @old_list = sort { $a <=> $b } @objects_with_greater_benefits;
 				my @new_list = sort { $a <=> $b } keys %{$objects_desired_by_this{$person}};
 				my @best_3_objects = ( $Opt01ObjForPersonI, $Opt02ObjForPersonI, $Opt03ObjForPersonI );
+				my $add_msg = " ";
+				$add_msg = sprintf "; \$BadValForPersonI = %.5f", $BadValForPersonI if ( $epsilon_scaling == 6 and $iter_count_local <= $epsilon_scaling and @objects_with_greater_benefits > $epsilon_scaling );
 
-				printf "<> PersonI = %3s ; %3s objects desired by this person (old list) = (@old_list) ; objects whose prices are still updated = (@updated_price) : %2s >= 1 ? \n", $person, scalar @old_list, scalar @updated_price;
-				printf "<> PersonI = %3s ; %3s objects desired by this person (new list) = (@new_list) ; \@best_3_objects = (@best_3_objects) \n", $person, scalar @new_list if ( defined $Opt03ObjForPersonI );
+				printf "<> PersonI = %3s ; %3s objects desired by this person (old list) = (@old_list) ; objects whose current values are still updated = (@updated_price) : %2s >= 1 ? \n", $person, scalar @old_list, scalar @updated_price;
+				printf "<> PersonI = %3s ; %3s objects desired by this person (new list) = (@new_list) ; \@best_3_objects = (@best_3_objects) $add_msg\n", $person, scalar @new_list if ( defined $Opt03ObjForPersonI );
 				printf "<> PersonI = %3s chose ObjectJ = %3s ; \$bid01ForPersonI %10.5f = \$Opt01ValForPersonI %.5f - \$Opt02ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt01ObjForPersonI, $bid01ForPersonI, $Opt01ValForPersonI, $Opt02ValForPersonI, $epsilon;
 				printf "<> PersonI = %3s chose ObjectJ = %3s ; \$bid02ForPersonI %10.5f = \$Opt02ValForPersonI %.5f - \$Opt03ValForPersonI %.5f + \$epsilon %.5f \n", $person, $Opt02ObjForPersonI, $bid02ForPersonI, $Opt02ValForPersonI, $Opt03ValForPersonI, $epsilon if ( defined $bid02ForPersonI );
 			}
@@ -796,7 +848,7 @@ __END__
 =head1 AUTHOR
 
     Claudio Fernandes de Souza Rodrigues
-	April 6, 2018
+	April 11, 2018
 	Sao Paulo, Brasil
 	claudiofsr@yahoo.com
 	
